(function (ng) {
    /**
     * @ngdoc module
     * @module apeman-ui-contrib-angular-tabbing
     * @name apeman-ui-contrib-angular-tabbing
     * @description
     * apeman ui module for angular to work with tabs.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-tabbing', [
            'apeman-ui-contrib-angular-objecting',
            'apeman-ui-contrib-angular-locating'
        ]);
})(angular);(function (ng) {

    /**
     * @ngdoc object
     * @module apeman-ui-contrib-angular-tabbing
     * @name apTabbingTmplConst
     * @description
     * Constant for template.
     */
    "use strict";

    ng
        .module('apeman-ui-contrib-angular-tabbing')
        .constant('apTabbingTmplConst',
        {
		    "templates/ap-tab-bar.html": "<div class=\"ap-tab-bar-item\"\n     data-ap-tab=\"{{item.key}}\"\n     ng:repeat=\"item in tab.items\"\n     ng:class=\"{'ap-tab-bar-item-selected': tab.isSelected(item.key)}\"\n     ng:click=\"tab.select(item.key);\"\n     ng:hide=\"item.hidden\">\n    <i class=\"ap-tab-bar-item-icon\"\n       ng:class=\"::item.icon\"\n       ng:show=\"!tab.isSelected(item.key)\"\n            ></i><i class=\"ap-tab-bar-item-icon ap-tab-bar-item-icon-selected\"\n                    ng:class=\"::(item.iconSelected || item.icon)\"\n                    ng:show=\"tab.isSelected(item.key)\"\n        ></i><label class=\"ap-tab-bar-item-label\">{{::item.title}}</label>\n</div>"
		}
        );

})(angular);(function (ng, $) {

    /**
     * @ngdoc directive
     * @name apTabBar
     * @module apeman-ui-contrib-angular-tabbing
     * @description Tab bar element.
     * @restrict E
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-tabbing')
        .directive('apTabBar', function (apTabbingTmplConst) {
            return {
                restrict: 'E',
                scope: {
                    tab: '=apTab'
                },
                link: function (scope, elm) {
                    var $elm = $(elm);
                    $elm.addClass('ap-tab-bar');
                },
                template: apTabbingTmplConst['templates/ap-tab-bar.html']
            }
        });

})(angular, jQuery);(function (ng) {
    /**
     * @ngdoc type
     * @name ApTab
     * @module apeman-ui-contrib-angular-tabbing
     * @description Tab context class.
     */
    "use strict";

    function _lowercase(str) {
        return String(str).toLowerCase();
    }

    function _uppercase(str) {
        return String(str).toUpperCase();
    }

    ng
        .module('apeman-ui-contrib-angular-tabbing')
        .factory('ApTab', function defineApTab(apTabSearch) {
            function ApTab(config) {
                var s = this;
                s.items = config.items || [];
                s._selected = null;

                var tabId = config.tabId || ('tab-' + Object.keys(ApTab.instances).length);
                s.setTabId(tabId);

                s.default = config.default || '';
                var selected = (apTabSearch.getSelected(tabId) || s.default).toUpperCase();
                if (selected) {
                    s.select(selected);
                }
            }

            ApTab.instances = {};

            ApTab.prototype = {
                tabId: null,
                /**
                 * Set view tab id.
                 * @param {string} TabId
                 */
                setTabId: function (TabId) {
                    var s = this;
                    var duplicate = !!ApTab.instances[TabId];
                    if (duplicate) {
                        throw new Error('Duplicate tab id:', TabId);
                    }
                    ApTab.instances[TabId] = s;
                    s.tabId = TabId;
                },
                /**
                 * Default tab name.
                 */
                default: '',
                /**
                 * Tab items.
                 */
                items: null,
                /**
                 * Select a tab.
                 * @param {string} key - Tab item key to select.
                 */
                select: function (key) {
                    var s = this,
                        isValid = s.isValid(key);
                    if (!isValid) {
                        console.warn('Invalid tab key: ' + key);
                        key = s.items[0] && s.items[0].key;
                    }
                    if (!key) {
                        return;
                    }
                    var prevTab = s._selected;
                    if (prevTab == key) {
                        return;
                    }
                    s._selected = _uppercase(key);

                    apTabSearch.setSelected(s.tabId, _lowercase(key));
                },
                /**
                 * Remove selected.
                 */
                deselect: function () {
                    var s = this;
                    s._selected = null;
                    apTabSearch.setSelected(s.tabId, '');
                },
                /**
                 * A tab item is selected or not.
                 * @param {string} key - Tab item key.
                 * @returns {boolean} - Selected or not.
                 */
                isSelected: function (key) {
                    var s = this;
                    return _uppercase(s._selected) === _uppercase(key);
                },
                isValid: function (key) {
                    var s = this,
                        items = s.items || [];
                    for (var i = 0, len = items.length; i < len; i++) {
                        var hit = _uppercase(items[i].key) === _uppercase(key);
                        if (hit) {
                            return true;
                        }
                    }
                    return false;
                }
            };

            return ApTab;
        });

})(angular);(function (ng) {

    /**
     * @ngdoc service
     * @module apeman-ui-contrib-angular-tabbing
     * @name apTabSearch
     * @description Reflect tab state with location search.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-tabbing')
        .service('apTabSearch', function ApViewTabSearch(apObjectNest,
                                                         apLocationSearch) {

            var s = this;

            /**
             * @ngdoc method
             * @name apTabSearch#setSelected
             * @param {string} tabId - Id for tabber.
             * @param {string} selected - Selected tab name.
             */
            s.setSelected = function setSelected(tabId, selected) {
                apLocationSearch.set(tabId, selected);
            };


            /**
             * @ngdoc method
             * @name apTabSearch#getSelected
             * @param {string} tabId - Id for tabber.
             * @returns {string}
             */
            s.getSelected = function getSelected(tabId) {
                return apLocationSearch.get(tabId);
            }


        });

})(angular);









(function (ng) {

    /**
     * @ngdoc service
     * @module apeman-ui-contrib-angular-tabbing
     * @name apTab
     * @description A tab instance service.
     */

    "use strict";

    ng
        .module('apeman-ui-contrib-angular-tabbing')
        .service('apTab', function apTabService(ApTab) {
            var s = this;
            s.__proto__ = new ApTab({});

            /**
             * Create a new ap tab instance.
             * @param {object} config
             * @returns {ApTab|*}
             */
            s.create = function (config) {
                return new ApTab(config);
            }
        });


})(angular);