(function (ng) {
    /**
     * @ngdoc type
     * @name ApTab
     * @module apeman-ui-contrib-angular-tabbing
     * @description Tab context class.
     */
    "use strict";

    function _lowercase(str) {
        return String(str).toLowerCase();
    }

    function _uppercase(str) {
        return String(str).toUpperCase();
    }

    ng
        .module('apeman-ui-contrib-angular-tabbing')
        .factory('ApTab', function defineApTab(apTabSearch) {
            function ApTab(config) {
                var s = this;
                s.items = config.items || [];
                s._selected = null;

                var tabId = config.tabId || ('tab-' + Object.keys(ApTab.instances).length);
                s.setTabId(tabId);

                s.default = config.default || '';
                var selected = (apTabSearch.getSelected(tabId) || s.default).toUpperCase();
                if (selected) {
                    s.select(selected);
                }
            }

            ApTab.instances = {};

            ApTab.prototype = {
                tabId: null,
                /**
                 * Set view tab id.
                 * @param {string} TabId
                 */
                setTabId: function (TabId) {
                    var s = this;
                    var duplicate = !!ApTab.instances[TabId];
                    if (duplicate) {
                        throw new Error('Duplicate tab id:', TabId);
                    }
                    ApTab.instances[TabId] = s;
                    s.tabId = TabId;
                },
                /**
                 * Default tab name.
                 */
                default: '',
                /**
                 * Tab items.
                 */
                items: null,
                /**
                 * Select a tab.
                 * @param {string} key - Tab item key to select.
                 */
                select: function (key) {
                    var s = this,
                        isValid = s.isValid(key);
                    if (!isValid) {
                        console.warn('Invalid tab key: ' + key);
                        key = s.items[0] && s.items[0].key;
                    }
                    if (!key) {
                        return;
                    }
                    var prevTab = s._selected;
                    if (prevTab == key) {
                        return;
                    }
                    s._selected = _uppercase(key);

                    apTabSearch.setSelected(s.tabId, _lowercase(key));
                },
                /**
                 * Remove selected.
                 */
                deselect: function () {
                    var s = this;
                    s._selected = null;
                    apTabSearch.setSelected(s.tabId, '');
                },
                /**
                 * A tab item is selected or not.
                 * @param {string} key - Tab item key.
                 * @returns {boolean} - Selected or not.
                 */
                isSelected: function (key) {
                    var s = this;
                    return _uppercase(s._selected) === _uppercase(key);
                },
                isValid: function (key) {
                    var s = this,
                        items = s.items || [];
                    for (var i = 0, len = items.length; i < len; i++) {
                        var hit = _uppercase(items[i].key) === _uppercase(key);
                        if (hit) {
                            return true;
                        }
                    }
                    return false;
                }
            };

            return ApTab;
        });

})(angular);